@@ -201,7 +201,8 @@ PUBLIC Bool XF86DRIOpenConnection(dpy, screen, hSAREA, busIdString)
 
     *hSAREA = rep.hSAREALow;
     if (sizeof(drm_handle_t) == 8) {
-       *hSAREA |= ((drm_handle_t) rep.hSAREAHigh) << 32;
+       const int shift = 32; /* var to prevent warning on next line */
+       *hSAREA |= ((drm_handle_t) rep.hSAREAHigh) << shift;
     }
 
     if (rep.length) {
@@ -563,7 +564,8 @@ PUBLIC Bool XF86DRIGetDeviceInfo(dpy, screen, hFrameBuffer,
 
     *hFrameBuffer = rep.hFrameBufferLow;
     if (sizeof(drm_handle_t) == 8) {
-       *hFrameBuffer |= ((drm_handle_t) rep.hFrameBufferHigh) << 32;
+       const int shift = 32; /* var to prevent warning on next line */
+       *hFrameBuffer |= ((drm_handle_t) rep.hFrameBufferHigh) << shift;
     }
 
     *fbOrigin = rep.framebufferOrigin;
