@@ -1,6 +1,6 @@
 /*
  * Mesa 3-D graphics library
- * Version:  6.4
+ * Version:  6.5
  *
  * Copyright (C) 1999-2005  Brian Paul   All Rights Reserved.
  *
@@ -34,13 +34,11 @@
 #include "glheader.h"
 #include "colormac.h"
 #include "context.h"
-#include "nvfragprog.h"
-#include "macros.h"
+#include "program_instruction.h"
 #include "program.h"
 
 #include "s_nvfragprog.h"
 #include "s_span.h"
-#include "s_texture.h"
 
 
 /* if 1, print some debugging info */
@@ -57,7 +55,7 @@ fetch_texel( GLcontext *ctx, const GLfloat texcoord[4], GLfloat lambda,
    SWcontext *swrast = SWRAST_CONTEXT(ctx);
 
    /* XXX use a float-valued TextureSample routine here!!! */
-   swrast->TextureSample[unit](ctx, unit, ctx->Texture.Unit[unit]._Current,
+   swrast->TextureSample[unit](ctx, ctx->Texture.Unit[unit]._Current,
                                1, (const GLfloat (*)[4]) texcoord,
                                &lambda, &rgba);
    color[0] = CHAN_TO_FLOAT(rgba[0]);
@@ -90,7 +88,7 @@ fetch_texel_deriv( GLcontext *ctx, const GLfloat texcoord[4],
                                          texcoord[0], texcoord[1], texcoord[3],
                                          1.0F / texcoord[3]);
 
-   swrast->TextureSample[unit](ctx, unit, ctx->Texture.Unit[unit]._Current,
+   swrast->TextureSample[unit](ctx, ctx->Texture.Unit[unit]._Current,
                                1, (const GLfloat (*)[4]) texcoord,
                                &lambda, &rgba);
    color[0] = CHAN_TO_FLOAT(rgba[0]);
@@ -106,7 +104,7 @@ fetch_texel_deriv( GLcontext *ctx, const GLfloat texcoord[4],
  */
 static INLINE const GLfloat *
 get_register_pointer( GLcontext *ctx,
-                      const struct fp_src_register *source,
+                      const struct prog_src_register *source,
                       const struct fp_machine *machine,
                       const struct fragment_program *program )
 {
@@ -136,8 +134,8 @@ get_register_pointer( GLcontext *ctx,
       case PROGRAM_STATE_VAR:
          /* Fallthrough */
       case PROGRAM_NAMED_PARAM:
-         ASSERT(source->Index < (GLint) program->Parameters->NumParameters);
-         src = program->Parameters->ParameterValues[source->Index];
+         ASSERT(source->Index < (GLint) program->Base.Parameters->NumParameters);
+         src = program->Base.Parameters->ParameterValues[source->Index];
          break;
       default:
          _mesa_problem(ctx, "Invalid input register file %d in fetch_vector4", source->File);
@@ -153,7 +151,7 @@ get_register_pointer( GLcontext *ctx,
  */
 static void
 fetch_vector4( GLcontext *ctx,
-               const struct fp_src_register *source,
+               const struct prog_src_register *source,
                const struct fp_machine *machine,
                const struct fragment_program *program,
                GLfloat result[4] )
@@ -194,7 +192,7 @@ fetch_vector4( GLcontext *ctx,
  */
 static GLboolean
 fetch_vector4_deriv( GLcontext *ctx,
-                     const struct fp_src_register *source,
+                     const struct prog_src_register *source,
                      const struct sw_span *span,
                      char xOrY, GLint column, GLfloat result[4] )
 {
@@ -323,7 +321,7 @@ fetch_vector4_deriv( GLcontext *ctx,
  */
 static void
 fetch_vector1( GLcontext *ctx,
-               const struct fp_src_register *source,
+               const struct prog_src_register *source,
                const struct fp_machine *machine,
                const struct fragment_program *program,
                GLfloat result[4] )
@@ -387,13 +385,13 @@ test_cc(GLuint condCode, GLuint ccMaskRule)
  * set-condition-code flags.
  */
 static void
-store_vector4( const struct fp_instruction *inst,
+store_vector4( const struct prog_instruction *inst,
                struct fp_machine *machine,
                const GLfloat value[4] )
 {
-   const struct fp_dst_register *dest = &(inst->DstReg);
-   const GLboolean clamp = inst->Saturate;
-   const GLboolean updateCC = inst->UpdateCondRegister;
+   const struct prog_dst_register *dest = &(inst->DstReg);
+   const GLboolean clamp = inst->SaturateMode == SATURATE_ZERO_ONE;
+   const GLboolean updateCC = inst->CondUpdate;
    GLfloat *dstReg;
    GLfloat dummyReg[4];
    GLfloat clampedValue[4];
@@ -497,7 +495,7 @@ init_machine_deriv( GLcontext *ctx,
    }
 
    /* Add derivatives */
-   if (program->InputsRead & (1 << FRAG_ATTRIB_WPOS)) {
+   if (program->Base.InputsRead & (1 << FRAG_ATTRIB_WPOS)) {
       GLfloat *wpos = (GLfloat*) machine->Inputs[FRAG_ATTRIB_WPOS];
       if (xOrY == 'X') {
          wpos[0] += 1.0F;
@@ -512,7 +510,7 @@ init_machine_deriv( GLcontext *ctx,
          wpos[3] += span->dwdy;
       }
    }
-   if (program->InputsRead & (1 << FRAG_ATTRIB_COL0)) {
+   if (program->Base.InputsRead & (1 << FRAG_ATTRIB_COL0)) {
       GLfloat *col0 = (GLfloat*) machine->Inputs[FRAG_ATTRIB_COL0];
       if (xOrY == 'X') {
          col0[0] += span->drdx * (1.0F / CHAN_MAXF);
@@ -527,7 +525,7 @@ init_machine_deriv( GLcontext *ctx,
          col0[3] += span->dady * (1.0F / CHAN_MAXF);
       }
    }
-   if (program->InputsRead & (1 << FRAG_ATTRIB_COL1)) {
+   if (program->Base.InputsRead & (1 << FRAG_ATTRIB_COL1)) {
       GLfloat *col1 = (GLfloat*) machine->Inputs[FRAG_ATTRIB_COL1];
       if (xOrY == 'X') {
          col1[0] += span->dsrdx * (1.0F / CHAN_MAXF);
@@ -542,7 +540,7 @@ init_machine_deriv( GLcontext *ctx,
          col1[3] += 0.0; /*XXX fix */
       }
    }
-   if (program->InputsRead & (1 << FRAG_ATTRIB_FOGC)) {
+   if (program->Base.InputsRead & (1 << FRAG_ATTRIB_FOGC)) {
       GLfloat *fogc = (GLfloat*) machine->Inputs[FRAG_ATTRIB_FOGC];
       if (xOrY == 'X') {
          fogc[0] += span->dfogdx;
@@ -552,7 +550,7 @@ init_machine_deriv( GLcontext *ctx,
       }
    }
    for (u = 0; u < ctx->Const.MaxTextureCoordUnits; u++) {
-      if (program->InputsRead & (1 << (FRAG_ATTRIB_TEX0 + u))) {
+      if (program->Base.InputsRead & (1 << (FRAG_ATTRIB_TEX0 + u))) {
          GLfloat *tex = (GLfloat*) machine->Inputs[FRAG_ATTRIB_TEX0 + u];
          /* XXX perspective-correct interpolation */
          if (xOrY == 'X') {
@@ -601,7 +599,7 @@ execute_program( GLcontext *ctx,
 #endif
 
    for (pc = 0; pc < maxInst; pc++) {
-      const struct fp_instruction *inst = program->Instructions + pc;
+      const struct prog_instruction *inst = program->Base.Instructions + pc;
 
       if (ctx->FragmentProgram.CallbackEnabled &&
           ctx->FragmentProgram.Callback) {
@@ -611,7 +609,7 @@ execute_program( GLcontext *ctx,
       }
 
       switch (inst->Opcode) {
-         case FP_OPCODE_ABS:
+         case OPCODE_ABS:
             {
                GLfloat a[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -622,7 +620,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_ADD:
+         case OPCODE_ADD:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -634,7 +632,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_CMP:
+         case OPCODE_CMP:
             {
                GLfloat a[4], b[4], c[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -647,7 +645,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_COS:
+         case OPCODE_COS:
             {
                GLfloat a[4], result[4];
                fetch_vector1( ctx, &inst->SrcReg[0], machine, program, a );
@@ -655,7 +653,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_DDX: /* Partial derivative with respect to X */
+         case OPCODE_DDX: /* Partial derivative with respect to X */
             {
                GLfloat a[4], aNext[4], result[4];
                struct fp_machine dMachine;
@@ -681,7 +679,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_DDY: /* Partial derivative with respect to Y */
+         case OPCODE_DDY: /* Partial derivative with respect to Y */
             {
                GLfloat a[4], aNext[4], result[4];
                struct fp_machine dMachine;
@@ -700,7 +698,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_DP3:
+         case OPCODE_DP3:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -714,7 +712,7 @@ execute_program( GLcontext *ctx,
 #endif
             }
             break;
-         case FP_OPCODE_DP4:
+         case OPCODE_DP4:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -728,7 +726,7 @@ execute_program( GLcontext *ctx,
 #endif
             }
             break;
-         case FP_OPCODE_DPH:
+         case OPCODE_DPH:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -738,7 +736,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_DST: /* Distance vector */
+         case OPCODE_DST: /* Distance vector */
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -750,7 +748,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_EX2: /* Exponential base 2 */
+         case OPCODE_EX2: /* Exponential base 2 */
             {
                GLfloat a[4], result[4];
                fetch_vector1( ctx, &inst->SrcReg[0], machine, program, a );
@@ -759,7 +757,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_FLR:
+         case OPCODE_FLR:
             {
                GLfloat a[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -770,7 +768,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_FRC:
+         case OPCODE_FRC:
             {
                GLfloat a[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -781,7 +779,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_KIL_NV: /* NV_f_p only */
+         case OPCODE_KIL_NV: /* NV_f_p only */
             {
                const GLuint swizzle = inst->DstReg.CondSwizzle;
                const GLuint condMask = inst->DstReg.CondMask;
@@ -793,7 +791,7 @@ execute_program( GLcontext *ctx,
                }
             }
             break;
-         case FP_OPCODE_KIL: /* ARB_f_p only */
+         case OPCODE_KIL: /* ARB_f_p only */
             {
                GLfloat a[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -802,7 +800,7 @@ execute_program( GLcontext *ctx,
                }
             }
             break;
-         case FP_OPCODE_LG2:  /* log base 2 */
+         case OPCODE_LG2:  /* log base 2 */
             {
                GLfloat a[4], result[4];
                fetch_vector1( ctx, &inst->SrcReg[0], machine, program, a );
@@ -811,7 +809,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_LIT:
+         case OPCODE_LIT:
             {
                const GLfloat epsilon = 1.0F / 256.0F; /* from NV VP spec */
                GLfloat a[4], result[4];
@@ -823,12 +821,20 @@ execute_program( GLcontext *ctx,
                result[0] = 1.0F;
                result[1] = a[0];
                /* XXX we could probably just use pow() here */
-               result[2] = (a[0] > 0.0F) ? (GLfloat) exp(a[3] * log(a[1])) : 0.0F;
+               if (a[0] > 0.0F) {
+                  if (a[1] == 0.0 && a[3] == 0.0)
+                     result[2] = 1.0;
+                  else
+                     result[2] = EXPF(a[3] * LOGF(a[1]));
+               }
+               else {
+                  result[2] = 0.0;
+               }
                result[3] = 1.0F;
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_LRP:
+         case OPCODE_LRP:
             {
                GLfloat a[4], b[4], c[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -841,7 +847,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_MAD:
+         case OPCODE_MAD:
             {
                GLfloat a[4], b[4], c[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -854,7 +860,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_MAX:
+         case OPCODE_MAX:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -872,7 +878,7 @@ execute_program( GLcontext *ctx,
 #endif
             }
             break;
-         case FP_OPCODE_MIN:
+         case OPCODE_MIN:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -884,7 +890,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_MOV:
+         case OPCODE_MOV:
             {
                GLfloat result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, result );
@@ -895,7 +901,7 @@ execute_program( GLcontext *ctx,
 #endif
             }
             break;
-         case FP_OPCODE_MUL:
+         case OPCODE_MUL:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -913,7 +919,7 @@ execute_program( GLcontext *ctx,
 #endif
             }
             break;
-         case FP_OPCODE_PK2H: /* pack two 16-bit floats in one 32-bit float */
+         case OPCODE_PK2H: /* pack two 16-bit floats in one 32-bit float */
             {
                GLfloat a[4], result[4];
                GLhalfNV hx, hy;
@@ -928,7 +934,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_PK2US: /* pack two GLushorts into one 32-bit float */
+         case OPCODE_PK2US: /* pack two GLushorts into one 32-bit float */
             {
                GLfloat a[4], result[4];
                GLuint usx, usy, *rawResult = (GLuint *) result;
@@ -942,7 +948,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_PK4B: /* pack four GLbytes into one 32-bit float */
+         case OPCODE_PK4B: /* pack four GLbytes into one 32-bit float */
             {
                GLfloat a[4], result[4];
                GLuint ubx, uby, ubz, ubw, *rawResult = (GLuint *) result;
@@ -960,7 +966,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_PK4UB: /* pack four GLubytes into one 32-bit float */
+         case OPCODE_PK4UB: /* pack four GLubytes into one 32-bit float */
             {
                GLfloat a[4], result[4];
                GLuint ubx, uby, ubz, ubw, *rawResult = (GLuint *) result;
@@ -978,7 +984,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_POW:
+         case OPCODE_POW:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector1( ctx, &inst->SrcReg[0], machine, program, a );
@@ -988,7 +994,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_RCP:
+         case OPCODE_RCP:
             {
                GLfloat a[4], result[4];
                fetch_vector1( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1003,7 +1009,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_RFL:
+         case OPCODE_RFL:
             {
                GLfloat axis[4], dir[4], result[4], tmp[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, axis );
@@ -1021,29 +1027,30 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_RSQ: /* 1 / sqrt() */
+         case OPCODE_RSQ: /* 1 / sqrt() */
             {
                GLfloat a[4], result[4];
                fetch_vector1( ctx, &inst->SrcReg[0], machine, program, a );
+               a[0] = FABSF(a[0]);
                result[0] = result[1] = result[2] = result[3] = INV_SQRTF(a[0]);
                store_vector4( inst, machine, result );
 #if DEBUG_FRAG
-               printf("RSQ %g = 1/sqrt(%g)\n", result[0], a[0]);
+               printf("RSQ %g = 1/sqrt(|%g|)\n", result[0], a[0]);
 #endif
             }
             break;
-         case FP_OPCODE_SCS: /* sine and cos */
+         case OPCODE_SCS: /* sine and cos */
             {
                GLfloat a[4], result[4];
                fetch_vector1( ctx, &inst->SrcReg[0], machine, program, a );
-               result[0] = (GLfloat)cos(a[0]);
-               result[1] = (GLfloat)sin(a[0]);
+               result[0] = (GLfloat)_mesa_cos(a[0]);
+               result[1] = (GLfloat)_mesa_sin(a[0]);
                result[2] = 0.0;  /* undefined! */
                result[3] = 0.0;  /* undefined! */
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SEQ: /* set on equal */
+         case OPCODE_SEQ: /* set on equal */
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1055,13 +1062,13 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SFL: /* set false, operands ignored */
+         case OPCODE_SFL: /* set false, operands ignored */
             {
                static const GLfloat result[4] = { 0.0F, 0.0F, 0.0F, 0.0F };
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SGE: /* set on greater or equal */
+         case OPCODE_SGE: /* set on greater or equal */
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1073,7 +1080,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SGT: /* set on greater */
+         case OPCODE_SGT: /* set on greater */
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1085,7 +1092,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SIN:
+         case OPCODE_SIN:
             {
                GLfloat a[4], result[4];
                fetch_vector1( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1094,7 +1101,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SLE: /* set on less or equal */
+         case OPCODE_SLE: /* set on less or equal */
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1106,7 +1113,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SLT: /* set on less */
+         case OPCODE_SLT: /* set on less */
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1118,7 +1125,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SNE: /* set on not equal */
+         case OPCODE_SNE: /* set on not equal */
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1130,13 +1137,13 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_STR: /* set true, operands ignored */
+         case OPCODE_STR: /* set true, operands ignored */
             {
                static const GLfloat result[4] = { 1.0F, 1.0F, 1.0F, 1.0F };
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SUB:
+         case OPCODE_SUB:
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1148,9 +1155,9 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_SWZ:
+         case OPCODE_SWZ:
             {
-               const struct fp_src_register *source = &inst->SrcReg[0];
+               const struct prog_src_register *source = &inst->SrcReg[0];
                const GLfloat *src = get_register_pointer(ctx, source,
                                                          machine, program);
                GLfloat result[4];
@@ -1171,7 +1178,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_TEX: /* Both ARB and NV frag prog */
+         case OPCODE_TEX: /* Both ARB and NV frag prog */
             /* Texel lookup */
             {
                GLfloat texcoord[4], color[4];
@@ -1192,10 +1199,14 @@ execute_program( GLcontext *ctx,
                fetch_texel( ctx, texcoord, 
 			    span->array->lambda[inst->TexSrcUnit][column],
 			    inst->TexSrcUnit, color );
+#if DEBUG_FRAG
+               if (color[3])
+                  printf("color[3] = %f\n", color[3]);
+#endif
                store_vector4( inst, machine, color );
             }
             break;
-         case FP_OPCODE_TXB: /* GL_ARB_fragment_program only */
+         case OPCODE_TXB: /* GL_ARB_fragment_program only */
             /* Texel lookup with LOD bias */
             {
                GLfloat texcoord[4], color[4], bias, lambda;
@@ -1211,7 +1222,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, color );
             }
             break;
-         case FP_OPCODE_TXD: /* GL_NV_fragment_program only */
+         case OPCODE_TXD: /* GL_NV_fragment_program only */
             /* Texture lookup w/ partial derivatives for LOD */
             {
                GLfloat texcoord[4], dtdx[4], dtdy[4], color[4];
@@ -1223,7 +1234,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, color );
             }
             break;
-         case FP_OPCODE_TXP: /* GL_ARB_fragment_program only */
+         case OPCODE_TXP: /* GL_ARB_fragment_program only */
             /* Texture lookup w/ projective divide */
             {
                GLfloat texcoord[4], color[4];
@@ -1251,12 +1262,12 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, color );
             }
             break;
-         case FP_OPCODE_TXP_NV: /* GL_NV_fragment_program only */
+         case OPCODE_TXP_NV: /* GL_NV_fragment_program only */
             /* Texture lookup w/ projective divide */
             {
                GLfloat texcoord[4], color[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, texcoord );
-               if (inst->TexSrcIdx != TEXTURE_CUBE_INDEX &&
+               if (inst->TexSrcTarget != TEXTURE_CUBE_INDEX &&
 		   texcoord[3] != 0.0) {
                   texcoord[0] /= texcoord[3];
                   texcoord[1] /= texcoord[3];
@@ -1268,7 +1279,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, color );
             }
             break;
-         case FP_OPCODE_UP2H: /* unpack two 16-bit floats */
+         case OPCODE_UP2H: /* unpack two 16-bit floats */
             {
                GLfloat a[4], result[4];
                const GLuint *rawBits = (const GLuint *) a;
@@ -1281,7 +1292,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_UP2US: /* unpack two GLushorts */
+         case OPCODE_UP2US: /* unpack two GLushorts */
             {
                GLfloat a[4], result[4];
                const GLuint *rawBits = (const GLuint *) a;
@@ -1294,7 +1305,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_UP4B: /* unpack four GLbytes */
+         case OPCODE_UP4B: /* unpack four GLbytes */
             {
                GLfloat a[4], result[4];
                const GLuint *rawBits = (const GLuint *) a;
@@ -1306,7 +1317,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_UP4UB: /* unpack four GLubytes */
+         case OPCODE_UP4UB: /* unpack four GLubytes */
             {
                GLfloat a[4], result[4];
                const GLuint *rawBits = (const GLuint *) a;
@@ -1318,7 +1329,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_XPD: /* cross product */
+         case OPCODE_XPD: /* cross product */
             {
                GLfloat a[4], b[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1330,7 +1341,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_X2D: /* 2-D matrix transform */
+         case OPCODE_X2D: /* 2-D matrix transform */
             {
                GLfloat a[4], b[4], c[4], result[4];
                fetch_vector4( ctx, &inst->SrcReg[0], machine, program, a );
@@ -1343,7 +1354,7 @@ execute_program( GLcontext *ctx,
                store_vector4( inst, machine, result );
             }
             break;
-         case FP_OPCODE_PRINT:
+         case OPCODE_PRINT:
             {
                if (inst->SrcReg[0].File != -1) {
                   GLfloat a[4];
@@ -1356,7 +1367,7 @@ execute_program( GLcontext *ctx,
                }
             }
             break;
-         case FP_OPCODE_END:
+         case OPCODE_END:
             return GL_TRUE;
          default:
             _mesa_problem(ctx, "Bad opcode %d in _mesa_exec_fragment_program",
@@ -1373,7 +1384,7 @@ init_machine( GLcontext *ctx, struct fp_machine *machine,
               const struct fragment_program *program,
               const struct sw_span *span, GLuint col )
 {
-   GLuint inputsRead = program->InputsRead;
+   GLuint inputsRead = program->Base.InputsRead;
    GLuint u;
 
    if (ctx->FragmentProgram.CallbackEnabled)
@@ -1446,8 +1457,8 @@ _swrast_exec_fragment_program( GLcontext *ctx, struct sw_span *span )
 
    ctx->_CurrentProgram = GL_FRAGMENT_PROGRAM_ARB; /* or NV, doesn't matter */
 
-   if (program->Parameters) {
-      _mesa_load_state_parameters(ctx, program->Parameters);
+   if (program->Base.Parameters) {
+      _mesa_load_state_parameters(ctx, program->Base.Parameters);
    }   
 
    for (i = 0; i < span->end; i++) {
@@ -1455,40 +1466,31 @@ _swrast_exec_fragment_program( GLcontext *ctx, struct sw_span *span )
          init_machine(ctx, &ctx->FragmentProgram.Machine,
                       ctx->FragmentProgram._Current, span, i);
 
-#ifdef USE_TCC
-         if (!_swrast_execute_codegen_program(ctx, program, ~0,
-					      &ctx->FragmentProgram.Machine,
-					      span, i)) {
-            span->array->mask[i] = GL_FALSE;  /* killed fragment */
-            span->writeAll = GL_FALSE;
-         }
-#else
          if (!execute_program(ctx, program, ~0,
                               &ctx->FragmentProgram.Machine, span, i)) {
             span->array->mask[i] = GL_FALSE;  /* killed fragment */
             span->writeAll = GL_FALSE;
          }
-#endif
 
          /* Store output registers */
          {
             const GLfloat *colOut
-               = ctx->FragmentProgram.Machine.Outputs[FRAG_OUTPUT_COLR];
+               = ctx->FragmentProgram.Machine.Outputs[FRAG_RESULT_COLR];
             UNCLAMPED_FLOAT_TO_CHAN(span->array->rgba[i][RCOMP], colOut[0]);
             UNCLAMPED_FLOAT_TO_CHAN(span->array->rgba[i][GCOMP], colOut[1]);
             UNCLAMPED_FLOAT_TO_CHAN(span->array->rgba[i][BCOMP], colOut[2]);
             UNCLAMPED_FLOAT_TO_CHAN(span->array->rgba[i][ACOMP], colOut[3]);
          }
          /* depth value */
-         if (program->OutputsWritten & (1 << FRAG_OUTPUT_DEPR)) {
+         if (program->Base.OutputsWritten & (1 << FRAG_RESULT_DEPR)) {
             const GLfloat depth
-               = ctx->FragmentProgram.Machine.Outputs[FRAG_OUTPUT_DEPR][2];
+               = ctx->FragmentProgram.Machine.Outputs[FRAG_RESULT_DEPR][2];
             span->array->z[i] = IROUND(depth * ctx->DrawBuffer->_DepthMaxF);
          }
       }
    }
 
-   if (program->OutputsWritten & (1 << FRAG_OUTPUT_DEPR)) {
+   if (program->Base.OutputsWritten & (1 << FRAG_RESULT_DEPR)) {
       span->interpMask &= ~SPAN_Z;
       span->arrayMask |= SPAN_Z;
    }
