@@ -1,8 +1,8 @@
 /*
  * Mesa 3-D graphics library
- * Version:  6.3
+ * Version:  6.5
  *
- * Copyright (C) 1999-2005  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2006  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -162,7 +162,7 @@ _swrast_culltriangle( GLcontext *ctx,
       return;								\
    }
 
-#define RENDER_SPAN( span  )						\
+#define RENDER_SPAN( span )						\
    GLuint i;								\
    span.intTex[0] -= FIXED_HALF; /* off-by-one error? */		\
    span.intTex[1] -= FIXED_HALF;					\
@@ -218,7 +218,7 @@ _swrast_culltriangle( GLcontext *ctx,
    span.intTex[0] -= FIXED_HALF; /* off-by-one error? */		\
    span.intTex[1] -= FIXED_HALF;					\
    for (i = 0; i < span.end; i++) {					\
-      const GLdepth z = FixedToDepth(span.z);				\
+      const GLuint z = FixedToDepth(span.z);				\
       if (z < zRow[i]) {						\
          GLint s = FixedToInt(span.intTex[0]) & smask;			\
          GLint t = FixedToInt(span.intTex[1]) & tmask;			\
@@ -544,7 +544,7 @@ affine_span(GLcontext *ctx, struct sw_span *span,
    info.twidth_log2 = obj->Image[0][b]->WidthLog2;			\
    info.smask = obj->Image[0][b]->Width - 1;				\
    info.tmask = obj->Image[0][b]->Height - 1;				\
-   info.format = obj->Image[0][b]->Format;				\
+   info.format = obj->Image[0][b]->_BaseFormat;				\
    info.filter = obj->MinFilter;					\
    info.envmode = unit->EnvMode;					\
    span.arrayMask |= SPAN_RGBA;						\
@@ -814,7 +814,7 @@ fast_persp_span(GLcontext *ctx, struct sw_span *span,
    info.twidth_log2 = obj->Image[0][b]->WidthLog2;			\
    info.smask = obj->Image[0][b]->Width - 1;				\
    info.tmask = obj->Image[0][b]->Height - 1;				\
-   info.format = obj->Image[0][b]->Format;				\
+   info.format = obj->Image[0][b]->_BaseFormat;				\
    info.filter = obj->MinFilter;					\
    info.envmode = unit->EnvMode;					\
 									\
@@ -904,14 +904,14 @@ fast_persp_span(GLcontext *ctx, struct sw_span *span,
  */
 #define NAME occlusion_zless_triangle
 #define INTERP_Z 1
-#define SETUP_CODE						\
-   struct gl_renderbuffer *rb					\
-      = ctx->DrawBuffer->Attachment[BUFFER_DEPTH].Renderbuffer;	\
-   ASSERT(ctx->Depth.Test);					\
-   ASSERT(!ctx->Depth.Mask);					\
-   ASSERT(ctx->Depth.Func == GL_LESS);				\
-   if (ctx->OcclusionResult && !ctx->Occlusion.Active) {	\
-      return;							\
+#define SETUP_CODE							\
+   struct gl_renderbuffer *rb = ctx->DrawBuffer->_DepthBuffer;		\
+   struct gl_query_object *q = ctx->Query.CurrentOcclusionObject;	\
+   ASSERT(ctx->Depth.Test);						\
+   ASSERT(!ctx->Depth.Mask);						\
+   ASSERT(ctx->Depth.Func == GL_LESS);					\
+   if (!q) {								\
+      return;								\
    }
 #define RENDER_SPAN( span )						\
    if (ctx->Visual.depthBits <= 16) {					\
@@ -919,10 +919,9 @@ fast_persp_span(GLcontext *ctx, struct sw_span *span,
       const GLushort *zRow = (const GLushort *)				\
          rb->GetPointer(ctx, rb, span.x, span.y);			\
       for (i = 0; i < span.end; i++) {					\
-         GLdepth z = FixedToDepth(span.z);				\
+         GLuint z = FixedToDepth(span.z);				\
          if (z < zRow[i]) {						\
-            ctx->OcclusionResult = GL_TRUE;				\
-            ctx->Occlusion.PassedCounter++;				\
+            q->Result++;						\
          }								\
          span.z += span.zStep;						\
       }									\
@@ -933,8 +932,7 @@ fast_persp_span(GLcontext *ctx, struct sw_span *span,
          rb->GetPointer(ctx, rb, span.x, span.y);			\
       for (i = 0; i < span.end; i++) {					\
          if ((GLuint)span.z < zRow[i]) {				\
-            ctx->OcclusionResult = GL_TRUE;				\
-            ctx->Occlusion.PassedCounter++;				\
+            q->Result++;						\
          }								\
          span.z += span.zStep;						\
       }									\
@@ -1057,7 +1055,7 @@ _swrast_choose_triangle( GLcontext *ctx )
       }
 
       /* special case for occlusion testing */
-      if ((ctx->Depth.OcclusionTest || ctx->Occlusion.Active) &&
+      if (ctx->Query.CurrentOcclusionObject &&
           ctx->Depth.Test &&
           ctx->Depth.Mask == GL_FALSE &&
           ctx->Depth.Func == GL_LESS &&
@@ -1074,7 +1072,7 @@ _swrast_choose_triangle( GLcontext *ctx )
          }
       }
 
-      if (ctx->Texture._EnabledCoordUnits || ctx->FragmentProgram._Active) {
+      if (ctx->Texture._EnabledCoordUnits || ctx->FragmentProgram._Active || ctx->ATIFragmentShader._Enabled) {
          /* Ugh, we do a _lot_ of tests to pick the best textured tri func */
 	 const struct gl_texture_object *texObj2D;
          const struct gl_texture_image *texImg;
@@ -1090,10 +1088,11 @@ _swrast_choose_triangle( GLcontext *ctx )
          /* First see if we can use an optimized 2-D texture function */
          if (ctx->Texture._EnabledCoordUnits == 0x1
              && !ctx->FragmentProgram._Active
+	     && !ctx->ATIFragmentShader._Enabled
              && ctx->Texture.Unit[0]._ReallyEnabled == TEXTURE_2D_BIT
              && texObj2D->WrapS == GL_REPEAT
 	     && texObj2D->WrapT == GL_REPEAT
-             && texObj2D->_IsPowerOfTwo
+             && texImg->_IsPowerOfTwo
              && texImg->Border == 0
              && texImg->Width == texImg->RowStride
              && (format == MESA_FORMAT_RGB || format == MESA_FORMAT_RGBA)
